import { promises as fsPromises } from 'fs';
import prompts from 'prompts';
import globalConfig from './globalConfig';
import { getConfig, configPath } from './config';
import ua from './ua';
import { getPortPromise } from 'portfinder';
import { createServer } from 'http';
import { createHttpTerminator, HttpTerminator } from 'http-terminator';
import opener from 'opener';

const cliLoginUrl = (port: number): string =>
  `${globalConfig.WEB_URL}/cli-login?port=${port}`;
const openCliLoginPage = (port: number): Promise<void> => {
  return new Promise((resolve) => {
    opener(cliLoginUrl(port), resolve);
  });
};
const cliRedirectUrl = (success: boolean): string =>
  `${globalConfig.WEB_URL}/cli-login-${success ? 'success' : 'error'}`;

const startServer = (
  port: number,
): Promise<{ tokenPromise: Promise<string> }> => {
  let httpTerminator: HttpTerminator = undefined;
  let resolveToken: (token: string) => void = undefined;
  const tokenPromise = new Promise<string>((resolve) => {
    resolveToken = resolve;
  });

  const server = createServer((req, res) => {
    const url = new URL('http://localhost' + req.url);
    if (req.method !== 'GET' || url.pathname !== '/') {
      res.statusCode = 400;
      res.end();
      return;
    }
    const token = url.searchParams.get('token');
    const tokenProvided = typeof token === 'string';
    res.setHeader('Location', cliRedirectUrl(tokenProvided));
    res.statusCode = 302;
    res.end();
    httpTerminator?.terminate();
    if (tokenProvided) {
      resolveToken(token);
    }
  });

  httpTerminator = createHttpTerminator({ server });

  return new Promise((resolve) => {
    server.listen(port, () => resolve({ tokenPromise }));
  });
};

export default async function authorization(): Promise<void | Record<
  string,
  any
>> {
  const config = await getConfig();

  if (config?.authToken) {
    return config;
  } else {
    const result = await prompts({
      type: 'text',
      name: 'response',
      message:
        "You don't appear to be logged in. Press any key to open up the browser to login or 'q' to exit:",
    });
    if (result.response === 'q') {
      throw new Error('Exiting.');
    }

    try {
      const port = await getPortPromise();
      const { tokenPromise } = await startServer(port);
      console.log('Waiting for login in the browser...');
      await openCliLoginPage(port);
      const conf = { authToken: await tokenPromise };
      console.log('Successfully logged in.');
      try {
        await fsPromises.writeFile(configPath, JSON.stringify(conf, null, 2));
        ua.event(
          {
            ec: 'user',
            ea: 'login',
            el: 'CLI',
          },
          () => conf,
        );
      } catch (e) {
        return e;
      }
    } catch (error) {
      let errorMessage = error.response.data.error?.code;
      if (errorMessage === 'EMAIL_PASSWORD_INCORRECT') {
        errorMessage = 'Sign in failed. Please check your email or password.';
      }
      throw Error(errorMessage);
    }
  }
}
