#!/usr/bin/env node
import fs from 'fs';
import { promises as fsPromises } from 'fs';
import { argv } from 'yargs';
import path from 'path';
import simpleGit, { SimpleGitOptions } from 'simple-git';
import authorization from './auth';
import transmit from './transmit';
import { configPath } from './config';

const run = async (): Promise<void> => {
  if (argv.h || argv.help) {
    const help = await fsPromises.readFile(
      path.resolve(__dirname, '../help.txt'),
      'utf-8',
    );

    console.log(help);
  } else if (argv.v || argv.version) {
    const npmPackage = JSON.parse(
      await fsPromises.readFile(
        path.resolve(__dirname, '../package.json'),
        'utf-8',
      ),
    );

    console.log(npmPackage.version);
  } else {
    for (const arg in argv) {
      if (arg) {
        switch (arg) {
          case '$0':
          case 'v':
          case 'version':
          case 'expires':
          case 'e':
          case 'signout':
          case 'h':
          case 'help':
          case '_':
            break;
          default:
            console.error(
              "The option: '" +
                arg +
                "' is not accepted. Please see help by running diffchecker --help.",
            );
            process.exit();
        }
      }
    }

    if (argv.signout) {
      fs.unlink(configPath, () => {
        console.log(
          "You've been signed out. Run the command again to sign in.",
        );
        process.exit();
      });
    }

    let fileOneExists = true;
    let fileTwoExists = true;

    try {
      await authorization();
      const fileOne = argv._[0] as string;
      const fileTwo = argv._[1] as string;
      try {
        await fsPromises.access(fileOne);
      } catch (error) {
        fileOneExists = false;
      }
      try {
        await fsPromises.access(fileTwo);
      } catch (error) {
        fileTwoExists = false;
      }

      /* Check if there's just one argument. If there is, assume user wants to compare with most recent git commit. Otherwise, read the next argument as a file to compare with. */
      if (argv._.length === 1 && fileOneExists) {
        const options: SimpleGitOptions = {
          baseDir: process.cwd(),
          binary: 'git',
          maxConcurrentProcesses: 6,
        };
        try {
          const git = simpleGit(options);
          const gitShow = await git.show([`HEAD:${fileOne}`]);
          transmit({
            left: gitShow,
            right: await fsPromises.readFile(fileOne, 'utf-8'),
          });
        } catch (error) {
          console.error(
            "Tried to look for a git version of that file, but couldn't locate a git repository nearby.",
          );
        }
      } else if (!fileOneExists) {
        console.error('File(s) not found. Please check your file paths.');
      } else if (fileOneExists && fileTwoExists) {
        transmit({
          left: await fsPromises.readFile(fileOne, 'utf-8'),
          right: await fsPromises.readFile(fileTwo, 'utf-8'),
        });
      } else {
        console.error(
          fileOneExists && fileTwoExists
            ? 'Something went wrong with your command.'
            : 'File(s) not found. Please check your file paths.',
        );
      }
    } catch (error) {
      console.error(error.message);
    }
  }
};
run();
